@@ -21,10 +21,11 @@ import sys
 import ldap
 import psycopg2
 import yaml
+from pkg_resources import get_distribution
 
-from . import __version__
 from .privilege import Privilege
 from .privilege import process_definitions as process_privileges
+from .psql import libpq_version
 from .utils import (
     deepget,
     deepset,
@@ -37,6 +38,9 @@ from .defaults import make_well_known_privileges
 from .defaults import shared_queries as default_shared_queries
 
 
+__dist__ = get_distribution('ldap2pg')
+__version__ = __dist__.version
+
 logger = logging.getLogger(__name__)
 
 
@@ -79,14 +83,22 @@ class ColoredStreamHandler(logging.StreamHandler):
 
 class VersionAction(_VersionAction):
     def __call__(self, parser, *a):
+        try:
+            libpqv = libpq_version()
+        except Exception:  # pragma: nocover
+            libpqv = 'unknown'
+        else:
+            libpqv = self.format_pq_version(libpqv)
+
         version = (
             "%(package)s %(version)s\n"
-            "psycopg2 %(psycopg2version)s\n"
+            "psycopg2 %(psycopg2version)s libpq %(libpqversion)s\n"
             "python-ldap %(ldapversion)s\n"
             "Python %(pyversion)s\n"
         ) % dict(
             package=__package__,
             version=__version__,
+            libpqversion=libpqv,
             psycopg2version=psycopg2.__version__,
             pyversion=sys.version,
             ldapversion=ldap.__version__,
@@ -95,6 +107,16 @@ class VersionAction(_VersionAction):
         print(version.strip())
         parser.exit()
 
+    @staticmethod
+    def format_pq_version(version):
+        pqnums = [
+            version / 10000,
+            version % 100,
+        ]
+        if version <= 100000:
+            pqnums[1:1] = [(version % 10000) / 100]
+        return '.'.join(str(int(n)) for n in pqnums)
+
 
 def define_arguments(parser):
     parser.add_argument(
@@ -324,17 +346,25 @@ def check_yaml_gotchas(file_config):
 
 
 class Configuration(dict):
+    minimum_version = 0
+
     DEFAULTS = {
+        # Default version correspond to ldap2pg 5.6, the first version of
+        # ldap2pg to have file version check. The minimum supported version is
+        # define in Configure.minimum_version attribute.
+        'version': 5,
         'check': False,
         'dry': True,
         'verbose': None,
         'verbosity': 'INFO',
         'color': False,
         'ldap': {
+            # See ldap2pg.ldap:gather_options function for default values.
             'uri': None,
             'host': None,
             'port': None,
             'binddn': None,
+            'starttls': None,
             'user': None,
             'password': None,
             'referrals': None,
@@ -388,15 +418,18 @@ class Configuration(dict):
         Mapping('color'),
         Mapping('check'),
         Mapping('dry'),
-        Mapping('verbose', env=[]),
+        Mapping('verbose', env=None),
         Mapping('verbosity', processor=V.verbosity),
-        Mapping('ldap:uri'),
-        Mapping('ldap:host'),
-        Mapping('ldap:port'),
-        Mapping('ldap:binddn', env=['LDAPBINDDN', 'LDAP_BIND']),
-        Mapping('ldap:user'),
-        Mapping('ldap:password', secret=True),
-        Mapping('ldap:referrals'),
+        Mapping('version', env=None),
+        # Don't read LDAP* env vars heres. ldap2pg.ldap:gather_options does it.
+        Mapping('ldap:uri', env=None),
+        Mapping('ldap:host', env=None),
+        Mapping('ldap:port', env=None),
+        Mapping('ldap:starttls', env=None),
+        Mapping('ldap:binddn', env=None),
+        Mapping('ldap:user', env=None),
+        Mapping('ldap:password', secret=True, env=None),
+        Mapping('ldap:referrals', env=None),
         Mapping(
             'postgres:dsn', env='PGDSN',
             secret=r'(?:password=|:[^/][^/].*@)',
@@ -421,8 +454,11 @@ class Configuration(dict):
 
     _file_candidates = [
         './ldap2pg.yml',
+        './ldap2pg.yaml',
         '~/.config/ldap2pg.yml',
+        '~/.config/ldap2pg.yaml',
         '/etc/ldap2pg.yml',
+        '/etc/ldap2pg.yaml',
     ]
 
     def find_filename(self, environ=os.environ, args=None):
@@ -467,8 +503,8 @@ class Configuration(dict):
     By default, ldap2pg runs in dry mode.
     """)
 
-    def has_ldap_query(self):
-        return [m['ldap'] for m in self['sync_map'] if 'ldap' in m]
+    def has_ldapsearch(self):
+        return any(True for m in self['sync_map'] if 'ldapsearch' in m)
 
     def bootstrap(self, environ=os.environ):
         debug = environ.get('DEBUG', '').lower() in ('1', 'y')
@@ -488,16 +524,7 @@ class Configuration(dict):
     def load(self, argv=None):
         # argv processing.
         logger.debug("Processing CLI arguments.")
-        parser = ArgumentParser(
-            add_help=False,
-            # Only store value from argv. Defaults are managed by
-            # Configuration.
-            argument_default=SUPPRESS_ARG,
-            description="PostgreSQL roles and privileges management.",
-            epilog=self.EPILOG,
-        )
-        define_arguments(parser)
-        args = parser.parse_args(sys.argv[1:] if argv is None else argv)
+        args = self.read_argv(argv)
 
         # Setup logging before parsing options. Reset verbosity with env var,
         # and compute verbosity from cumulated args.
@@ -532,20 +559,34 @@ class Configuration(dict):
         if not self.get('debug'):
             sys.stdin.close()
 
-        check_yaml_gotchas(file_config)
-        self.warn_unknown_config(file_config)
-
         # Now merge all config sources.
-        default_privileges = make_well_known_privileges()
         try:
             self.merge(file_config=file_config, environ=os.environ, args=args)
-            postprocess_privilege_options(self, default_privileges)
         except ValueError as e:
             raise ConfigurationError("Failed to load configuration: %s" % (e,))
 
         logger.debug("Configuration loaded.")
 
+    def read_argv(self, argv=None):
+        parser = ArgumentParser(
+            add_help=False,
+            # Only store value from argv. Defaults are managed by
+            # Configuration.
+            argument_default=SUPPRESS_ARG,
+            description="PostgreSQL roles and privileges management.",
+            epilog=self.EPILOG,
+        )
+        define_arguments(parser)
+        return parser.parse_args(sys.argv[1:] if argv is None else argv)
+
     def merge(self, file_config, environ=os.environ, args=object()):
+        if file_config.get('version', 5) < self.minimum_version:
+            raise UserError(
+                "File version %(version)s is not supported." % file_config)
+
+        check_yaml_gotchas(file_config)
+        self.warn_unknown_config(file_config)
+
         for mapping in self.MAPPINGS:
             value = mapping.process(
                 default=deepget(self, mapping.path),
@@ -558,13 +599,20 @@ class Configuration(dict):
         if self['verbose'] is not None:
             self['verbosity'] = 'DEBUG' if self['verbose'] else 'INFO'
 
-    def read(self, fo, name, mode):
+        extract_static_rules(self)
+        default_privileges = make_well_known_privileges()
+        postprocess_privilege_options(self, default_privileges)
+
+    def read(self, fo, name, mode=0o400):
         try:
             payload = yaml.safe_load(fo)
         except yaml.error.YAMLError as e:
             msg = "YAML error with %s: %s" % (name, e)
             raise ConfigurationError(msg)
 
+        return self.validate_raw_yaml(payload, name, mode)
+
+    def validate_raw_yaml(self, payload, name, mode=0o400):
         if payload is None:
             raise ConfigurationError("Configuration is empty.")
         if isinstance(payload, list):
@@ -621,3 +669,59 @@ class Configuration(dict):
 
             if k not in known_keys:
                 logger.warning("Unknown config entry: %s.", k)
+
+
+def extract_static_rules(config):
+    # This procedure extract static rules from mapping to avoid roles dropped
+    # or not created because an ldap search was empty.
+
+    sync_map = config.get('sync_map', [])
+    for i, mapping in enumerate(sync_map):
+        if "ldap" not in mapping:
+            continue
+
+        rules = mapping.get('roles', [])
+        for rule in reversed(rules[:]):
+            static_names = [fmt.spec for fmt in rule.names if fmt.static]
+            dynamic_names = [fmt for fmt in rule.names if not fmt.static]
+            if not static_names:
+                continue
+
+            static_rule = rule.copy(names=static_names)
+            if static_rule.is_dynamic:
+                # There is still LDAP attribute mapping in e.g. parent or
+                # member. Abort here static rule extracting.
+                continue
+
+            if dynamic_names:
+                # In case of mixed dynamic/static names, just edit in-place.
+                rule.names[:] = dynamic_names
+            else:
+                # Rule was only static, remove it from ldap mapping..
+                rules.remove(rule)
+
+            logger.debug("Extracted static role rule %s.", static_rule)
+            sync_map.insert(i, dict(roles=[static_rule]))
+
+        rules = mapping.get('grants', [])
+        for rule in reversed(rules[:]):
+            static_roles = [fmt.spec for fmt in rule.roles if fmt.static]
+            dynamic_roles = [fmt for fmt in rule.roles if not fmt.static]
+            if not static_roles:
+                continue
+
+            static_rule = rule.copy(roles=static_roles)
+            if static_rule.is_dynamic:
+                # There is still LDAP attribute mapping in e.g. parent or
+                # member. Abort here static rule extracting.
+                continue
+
+            if dynamic_roles:
+                # In case of mixed dynamic/static roles, just edit in-place.
+                rule.roles[:] = dynamic_roles
+            else:
+                # Rule was only static, remove it from ldap mapping..
+                rules.remove(rule)
+
+            logger.debug("Extracted static grant rule %s.", static_rule)
+            sync_map.insert(i, dict(grants=[static_rule]))
