@@ -55,19 +55,19 @@ class PostgresInspector(object):
 
     def row1(self, rows):
         # Just single value row for e.g databases, managed_roles, owners, etc.
-        for row in rows:
+        for row in handle_decoding_error(rows):
             yield row[0]
 
     def process_roles(self, rows):
         # all_roles query signatures: name, [members, [options ...]]
-        for row in rows:
+        for row in handle_decoding_error(rows):
             yield Role.from_row(*row)
 
     def process_grants(self, privilege, dbname, rows):
         # GRANT query signatures: schema, role, [<privilege options> ...]
         sql = str(privilege.grant_sql) + str(privilege.revoke_sql)
         schema_aware = '{schema}' in sql
-        for row in rows:
+        for row in handle_decoding_error(rows):
             if len(row) < 2:
                 fmt = "%s's inspect query doesn't return role as column 2"
                 raise UserError(fmt % (privilege,))
@@ -79,7 +79,7 @@ class PostgresInspector(object):
             yield Grant.from_row(privilege.name, dbname, *row)
 
     def process_schemas(self, rows):
-        for row in rows:
+        for row in handle_decoding_error(rows):
             if not isinstance(row, (list, tuple)):
                 row = [row]
 
@@ -199,6 +199,7 @@ class PostgresInspector(object):
 
     def fetch_me(self):
         with self.psql() as psql:
+            logger.debug("Introspecting session Postgres role.")
             return self.fetch(psql, self.inspect_me)[0]
 
     def fetch_roles(self):
@@ -206,7 +207,9 @@ class PostgresInspector(object):
         # roles. That's the minimum de synchronize roles.
 
         with self.psql() as psql:
+            logger.debug("Inspecting databases.")
             databases = self.fetch(psql, 'databases', self.row1)
+            logger.debug("Inspecting all defined roles in cluster.")
             pgallroles = RoleSet(self.fetch(
                 psql, self.format_roles_query(), self.process_roles))
             if not self.queries.get('managed_roles'):
@@ -245,8 +248,8 @@ class PostgresInspector(object):
                 else:
                     # False owner means schemas_query is not aware of owners.
                     if owners is None:
-                        logger.debug("Globally inspecting owners...")
                         with self.psql() as psql:
+                            logger.debug("Globally inspecting owners...")
                             owners = set(self.fetch(psql, 'owners', self.row1))
                     s_owners = owners
                 # Only filter if managedroles are defined. This allow ACL only
@@ -270,8 +273,10 @@ class PostgresInspector(object):
                     privilege)
                 continue
 
-            logger.debug("Searching GRANTs of privilege %s.", privilege)
             for dbname, psql in self.psql.itersessions(schemas):
+                logger.debug(
+                    "Searching GRANTs of privilege %s in %s.",
+                    privilege, dbname)
                 if isinstance(privilege.inspect, dict):
                     rows = self.fetch_shared_query(
                         name=privilege.inspect['shared_query'],
@@ -312,3 +317,13 @@ class PostgresInspector(object):
             r[1:] for r in self.query_cache[cache_key]
             if r[0] in keys
         ]
+
+
+def handle_decoding_error(iterator):
+    try:
+        for item in iterator:
+            yield item
+    except UnicodeDecodeError as e:
+        raise UserError(
+            "Encoding error: Can't decode %r from %s."
+            % (e.object, e.encoding))
