@@ -4,6 +4,12 @@ from codecs import open
 from collections import namedtuple
 import logging
 import os
+try:
+    from shlex import quote as shquote
+except ImportError:  # pragma: nocover_py3
+    def shquote(x):
+        return "'%s'" % x
+
 
 import ldap
 
@@ -16,6 +22,7 @@ except ImportError:  # pragma: nocover
 from ldap.dn import str2dn as native_str2dn
 from ldap import sasl
 
+from .format import FormatVars, FormatValue
 from .utils import decode_value, encode_value, PY2, uniq
 from .utils import Timer
 from .utils import UserError
@@ -37,7 +44,7 @@ if SCOPE_SUBORDINATE:
 
 SCOPES_STR = dict((v, k) for k, v in SCOPES.items())
 
-DN_COMPONENTS = ('dn', 'cn', 'l', 'st', 'o', 'ou', 'c', 'street', 'dc', 'uid')
+DN_COMPONENTS = ('cn', 'l', 'st', 'o', 'ou', 'c', 'street', 'dc', 'uid')
 
 
 def parse_scope(raw):
@@ -74,56 +81,129 @@ class RDNError(NameError):
         self.dn = dn
 
 
-def get_attribute(entry, attribute):
-    # Generate all values from entry for accessor attribute. Attribute can be a
-    # single attribute name, a path to a RDN in a distinguished name, or an
-    # attribute of a join (aka subquery).
-    _, attributes, joins = entry
-    path = attribute.lower().split('.')
-    try:
-        values = attributes[path[0]]
-    except KeyError:
-        raise ValueError("Unknown attribute %r" % (path[0],))
+class MissingAttributeError(KeyError):
+    def __str__(self):
+        return "Missing attribute: %s." % (self.args[0])
 
-    attribute = path[0]
-    path = path[1:]
-    if not path:
-        for value in values:
-            yield value
-    elif path[0] in DN_COMPONENTS:
-        for value in values:
-            raw_dn = value
-            try:
-                dn = str2dn(value)
-            except ValueError:
-                msg = "Can't parse DN from attribute %s=%s" % (
-                    attribute, value)
-                raise ValueError(msg)
-            value = dict()
-            for (type_, name, _), in dn:
-                value.setdefault(type_.lower(), name)
-            try:
-                yield value[path[0]]
-            except KeyError:
-                yield RDNError("Unknown RDN %s" % (path[0],), raw_dn)
-    else:
+
+class LDAPEntry(object):
+    __slot__ = (
+        'dn',
+        'attributes',
+        'children',
+        '__dict__',
+    )
+
+    def __init__(self, dn, attributes=None, children=None):
+        self.dn = dn
+        self.attributes = attributes or {}
+        self.children = children or {}
+
+    def __eq__(self, other):
+        return (
+            self.dn == other.dn
+            and self.attributes == other.attributes
+            and self.children == other.children
+        )
+
+    def __repr__(self):
+        return '<%s %s>' % (
+            self.__class__.__name__,
+            self.dn,
+        )
+
+    def __getitem__(self, key):
+        # Generate all values from entry for accessor attribute. Attribute can
+        # be a single attribute name, a path to a RDN in a distinguished name,
+        # or an attribute of a join (aka subquery).
+
+        if "dn" == key:
+            yield self.dn
+            return
+
+        path = key.lower().split('.')
+
+        # First level access.
         try:
-            joined_entries = joins[attribute]
+            values = self.attributes[path[0]]
+            path = path[1:]
         except KeyError:
-            msg = "Missing join result for %s" % (attribute,)
-            raise ValueError(msg)
+            # Fallback DN components in DN, just like having `dn.XX`.
+            if path[0] in DN_COMPONENTS:
+                values = [self.dn]
+            elif "dn" == path[0]:
+                values = [self.dn]
+                path = path[1:]
+            else:
+                raise MissingAttributeError(path[0])
 
-        for joined_entry in joined_entries:
-            for value in get_attribute(joined_entry, '.'.join(path)):
+        if not path:
+            for value in values:
                 yield value
+        elif path[0] in DN_COMPONENTS:
+            for value in values:
+                raw_dn = value
+                try:
+                    dn = str2dn(value)
+                except ValueError:
+                    msg = "Can't parse DN from attribute %s=%s." % (
+                        key, value)
+                    raise ValueError(msg)
+                value = dict()
+                for (type_, name, _), in dn:
+                    value.setdefault(type_.lower(), name)
+                try:
+                    yield value[path[0]]
+                except KeyError:
+                    yield RDNError("Unknown RDN %s." % (path[0],), raw_dn)
+        else:
+            raise MissingAttributeError(path[0])
 
+    def build_format_vars(self, map_, processor=None):
+        # Builds a dicts of values from self corresponding to the request
+        # described in map_.
 
-def lower_attributes(entry):
-    dn, attributes = entry
-    return dn, dict([
-        (k.lower(), v)
-        for k, v in attributes.items()
-    ])
+        if processor is None:
+            def processor(x):
+                return x
+
+        vars_ = FormatVars(map_)
+        for objname, attributes in map_.items():
+            prefix = ''
+            if objname in ("__self__", "dn"):
+                entries = [self]
+            else:
+                try:
+                    entries = self.children[objname]
+                except KeyError:
+                    # Accessing a RDN of a foreign-key like {member.cn}. Fake
+                    # sub-query entries for each DN as returned by
+                    # __self__.member.
+                    entries = [
+                        LDAPEntry(dn) for dn in processor(self[objname])
+                    ]
+
+            vars_[objname] = []
+            for entry in entries:
+                entry_vars = {}
+
+                for attr in attributes:
+                    fullattr = prefix + attr
+                    if attr.startswith("dn."):
+                        dn = entry_vars.get("dn")
+                        if not isinstance(dn, dict):
+                            dn = dict(dn=FormatValue(entry.dn))
+                            entry_vars["dn"] = [dn]
+                        dn[attr[3:]] = FormatValue(next(entry[fullattr]))
+                    else:
+                        entry_vars[attr] = [
+                            FormatValue(v) for v in processor(entry[fullattr])
+                        ]
+
+                entry_vars.setdefault("dn", [FormatValue(entry.dn)])
+                vars_[objname].append(entry_vars)
+
+        return vars_
 
 
 class EncodedParamsCallable(object):  # pragma: nocover_py3
@@ -159,9 +239,10 @@ class LDAPLogger(object):
 
     def search_s(self, base, scope, filter, attributes):
         logger.debug(
-            "Doing: ldapsearch%s -b %s -s %s '%s' %s",
+            "Doing: ldapsearch%s -b %s -s %s %s %s",
             self.connect_opts,
-            base, SCOPES_STR[scope], filter, ' '.join(attributes or []),
+            base, SCOPES_STR[scope], shquote(filter),
+            ' '.join(attributes or []),
         )
         with self.timer:
             return self.wrapped.search_s(base, scope, filter, attributes)
@@ -169,7 +250,7 @@ class LDAPLogger(object):
     def simple_bind_s(self, binddn, password):
         self.connect_opts = ' -x'
         if binddn:
-            self.connect_opts += ' -D %s' % (binddn,)
+            self.connect_opts += ' -D %s' % (shquote(binddn),)
         if password:
             self.connect_opts += ' -W'
         self.log_connect()
@@ -179,20 +260,20 @@ class LDAPLogger(object):
         self.connect_opts = ' -Y %s' % (auth.mech.decode('ascii'),)
         if sasl.CB_AUTHNAME in auth.cb_value_dict:
             self.connect_opts += ' -U %s' % (
-                auth.cb_value_dict[sasl.CB_AUTHNAME],)
+                shquote(auth.cb_value_dict[sasl.CB_AUTHNAME]),)
         if sasl.CB_PASS in auth.cb_value_dict:
             self.connect_opts += ' -W'
         self.log_connect()
         return self.wrapped.sasl_interactive_bind_s(who, auth, *a, **kw)
 
     def log_connect(self):
-        logger.debug("Doing: ldapwhoami%s", self.connect_opts)
+        logger.debug("Authenticating: ldapwhoami%s", self.connect_opts)
 
 
 def connect(**kw):
     # Sources order, see ldap.conf(3)
     #   variable     $LDAPNOINIT, and if that is not set:
-    #   system file  /etc/ldap/ldap.conf,
+    #   system file  /etc/ldap/ldap.conf, /etc/openldap/ldap.conf
     #   user files   $HOME/ldaprc,  $HOME/.ldaprc,  ./ldaprc,
     #   system file  $LDAPCONF,
     #   user files   $HOME/$LDAPRC, $HOME/.$LDAPRC, ./$LDAPRC,
@@ -208,6 +289,13 @@ def connect(**kw):
         conn = UnicodeModeLDAPObject(conn)
 
     conn = LDAPLogger(conn)
+    conn.set_option(ldap.OPT_NETWORK_TIMEOUT, 120)
+
+    if options.get('STARTTLS'):
+        logger.debug("Sending STARTTLS.")
+        conn.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
+        conn.start_tls_s()
+
     # Don't follow referrals by default. This is the behaviour of ldapsearch
     # and friends. Following referrals leads to strange errors with Active
     # directory. REFERRALS can still be activated through ldaprc, env var and
@@ -261,9 +349,23 @@ class Options(dict):
     parse_password = _parse_raw
     parse_sasl_mech = _parse_raw
     parse_referrals = _parse_bool
+    parse_starttls = _parse_bool
 
 
 def gather_options(environ=None, **kw):
+    # This is the main point for LDAP configuration marshall. kw is the ldap
+    # stanza in ldap2pg.yml.
+    #
+    # ldap2pg handles a subset of openldap libldap parameters. These parameters
+    # are declared in options variable with their default value as for ldap2pg.
+    # These parameters are used in ldap2pg client-side logic.
+    #
+    # Value for these parameters are searching in ldap.conf files, environment
+    # variables and YAML file.
+    default_conffiles = [
+        '/etc/openldap/ldap.conf',
+        '/etc/ldap/ldap.conf',
+    ]
     options = Options(
         URI='',
         HOST='',
@@ -272,6 +374,9 @@ def gather_options(environ=None, **kw):
         USER=None,
         PASSWORD='',
         SASL_MECH=None,
+        # This is an extension to ldap.conf. Equivalent of -Z CLI options of
+        # ldap-utils.
+        STARTTLS=False,
         REFERRALS=False,
     )
 
@@ -285,10 +390,12 @@ def gather_options(environ=None, **kw):
     if 'NOINIT' in environ:
         logger.debug("LDAPNOINIT defined. Disabled ldap.conf loading.")
     else:
-        for e in read_files(conf='/etc/ldap/ldap.conf', rc='ldaprc'):
+        for e in read_files(conf=default_conffiles, rc='ldaprc'):
             logger.debug('Read %s from %s.', e.option, e.filename)
             options.set_raw(e.option, e.value)
-        for e in read_files(conf=options.get('CONF'), rc=options.get('RC')):
+        customconf = environ.pop('CONF', options.get('CONF'))
+        customrc = environ.pop('RC', options.get('RC'))
+        for e in read_files(conf=customconf, rc=customrc):
             logger.debug('Read %s from %s.', e.option, e.filename)
             options.set_raw(e.option, e.value)
         for option, value in environ.items():
@@ -315,10 +422,12 @@ def gather_options(environ=None, **kw):
 
 def read_files(conf, rc):
     candidates = []
-    if conf:
+    if isinstance(conf, list):
+        candidates.extend(conf)
+    elif isinstance(conf, str):
         candidates.append(conf)
     if rc:
-        candidates.extend(['~/%s' % rc, '~/.%s' % rc, rc])
+        candidates.extend(['~/%s' % rc, '~/.%s' % rc, './%s' % rc])
     candidates = uniq(map(
         lambda p: os.path.realpath(os.path.expanduser(p)),
         candidates,
@@ -348,7 +457,12 @@ def parserc(fo):
         if line.startswith('#'):
             continue
 
-        option, value = line.split(None, 1)
+        try:
+            option, value = line.split(None, 1)
+        except ValueError:
+            raise UserError(
+                "Bad syntax in %s at line %s: %s" % (filename, lineno, line))
+
         yield RCEntry(
             filename=filename,
             lineno=lineno+1,
