@@ -2,10 +2,11 @@ from itertools import groupby
 import logging
 from fnmatch import fnmatch
 
+from .format import AttributesMap, FormatList, collect_fields
 from .psql import Query
 from .utils import (
-    AllDatabases, FormatList, UserError,
-    collect_fields, unicode, make_group_map,
+    AllDatabases, UserError,
+    unicode, make_group_map,
 )
 
 
@@ -229,9 +230,23 @@ class GrantRule(object):
         self.roles = FormatList.factory(roles)
         self.role_match = role_match
         self.all_fields = collect_fields(
-            self.privilege, self.databases, self.schemas, self.roles,
+            self.databases, self.privilege, self.roles, self.schemas,
         )
 
+    @property
+    def attributes_map(self):
+        map_ = AttributesMap()
+        for lst in self.databases, self.schemas, self.roles, self.privilege:
+            map_.update(lst.attributes_map)
+        return map_
+
+    @property
+    def is_dynamic(self):
+        return len(self.all_fields) > 0
+
+    def __eq__(self, other):
+        return self.as_dict() == other.as_dict()
+
     def __repr__(self):
         return '<%s %s on [%s].[%s] to [%s]>' % (
             self.__class__.__name__,
@@ -241,6 +256,10 @@ class GrantRule(object):
             self.roles,
         )
 
+    def copy(self, **kw):
+        kw = dict(self.as_dict(), **kw)
+        return self.__class__(**kw)
+
     def generate(self, vars_):
         privilege = next(self.privilege.expand(vars_))
         databases = list(self.databases.expand(vars_))
