@@ -1,5 +1,6 @@
 from __future__ import unicode_literals
 
+import ctypes
 import logging
 import re
 
@@ -16,9 +17,20 @@ from .utils import (
 )
 
 
-logger = logging.getLogger(__name__)
+class ChangeLogger(logging.getLoggerClass()):
+    def change(self, msg, *args, **kwargs):
+        if self.isEnabledFor(logging.CHANGE):
+            self._log(logging.CHANGE, msg, args, **kwargs)
 
 
+logging.CHANGE = logging.INFO + 5
+logging.addLevelName(logging.CHANGE, 'CHANGE')
+# Use ChangeLogger class only in this module.
+logging.setLoggerClass(ChangeLogger)
+logger = logging.getLogger(__name__)
+logging.setLoggerClass(logging.Logger)
+logger.change  # Raises AttributeError if logger is not ChangeLogger.
+
 psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
 psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)
 
@@ -30,7 +42,8 @@ def inject_database_in_connstring(connstring, dbname):
     if dbname is None:
         return connstring
 
-    if connstring.startswith('postgres://'):
+    if (connstring.startswith('postgres://') or
+            connstring.startswith('postgresql://')):
         pr = list(urlparse(connstring))
         pr[2] = dbname
         return urlunparse(pr)
@@ -158,7 +171,13 @@ class PSQLSession(object):
     def mogrify(self, qry, *a, **kw):
         qry = qry.encode('utf-8')
         sql = self.cursor.mogrify(qry, *a, **kw)
-        return sql.decode(self.conn.encoding)
+        target_encoding = psycopg2.extensions.encodings[self.conn.encoding]
+        try:
+            return sql.decode(target_encoding)
+        except UnicodeDecodeError:
+            raise UserError(
+                "Can't encode query to database encoding %s."
+                % self.conn.encoding)
 
 
 class Query(object):
@@ -195,3 +214,23 @@ def expandqueries(queries, databases):
     for query in queries:
         for single_query in query.expand(databases):
             yield single_query
+
+
+def libpq_version():
+    # Search libpq version bound to this process.
+
+    try:
+        return psycopg2.__libpq_version__
+    except AttributeError:
+        # Search for libpq.so path in loaded libraries.
+        with open('/proc/self/maps') as fo:
+            for line in fo:
+                values = line.split()
+                path = values[-1]
+                if '/libpq' in path:
+                    break
+            else:  # pragma: nocover
+                raise Exception("libpq.so not loaded")
+
+        libpq = ctypes.cdll.LoadLibrary(path)
+        return libpq.PQlibVersion()
